package com.github.polpetta.mezzotre.telegram.callbackquery;

import com.github.polpetta.mezzotre.orm.model.CallbackQueryContext;
import com.google.inject.assistedinject.Assisted;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import javax.inject.Inject;
import org.slf4j.Logger;

public class NotFound implements Processor {

  private final Logger log;
  private final String eventName;

  @Inject
  public NotFound(Logger logger, @Assisted String eventName) {
    this.log = logger;
    this.eventName = eventName;
  }

  @Override
  public String getEventName() {
    return "eventNotFound";
  }

  @Override
  public CompletableFuture<Optional<BaseRequest<?, ?>>> process(
      CallbackQueryContext callbackQueryContext, Update update) {
    log.warn(
        "A stray event was detected for callback "
            + callbackQueryContext.getId()
            + " event name "
            + eventName);
    return CompletableFuture.completedFuture(Optional.empty());
  }
}
