package com.github.polpetta.mezzotre.telegram.callbackquery;

import com.github.polpetta.mezzotre.orm.model.CallbackQueryContext;
import com.github.polpetta.mezzotre.telegram.model.Help;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.model.request.InlineKeyboardButton;
import com.pengrad.telegrambot.model.request.InlineKeyboardMarkup;
import com.pengrad.telegrambot.model.request.ParseMode;
import com.pengrad.telegrambot.request.BaseRequest;
import com.pengrad.telegrambot.request.EditMessageText;
import com.pengrad.telegrambot.request.SendMessage;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import javax.inject.Inject;
import javax.inject.Named;

class ShowHelp implements Processor {

  public static String EVENT_NAME = "showHelp";
  private final Executor threadPool;
  private final Help modelHelp;
  private final Map<String, com.github.polpetta.mezzotre.telegram.command.Processor>
      tgCommandProcessors;
  private final Map<String, Processor> eventProcessor;

  // FIXME tests
  @Inject
  public ShowHelp(
      @Named("eventThreadPool") Executor threadPool,
      com.github.polpetta.mezzotre.telegram.model.Help modelHelp,
      @Named("commandProcessor")
          Map<String, com.github.polpetta.mezzotre.telegram.command.Processor> tgCommandProcessors,
      @Named("eventProcessors")
          Map<String, com.github.polpetta.mezzotre.telegram.callbackquery.Processor>
              eventProcessor) {
    this.threadPool = threadPool;
    this.modelHelp = modelHelp;
    this.tgCommandProcessors = tgCommandProcessors;
    this.eventProcessor = eventProcessor;
  }

  @Override
  public String getEventName() {
    return EVENT_NAME;
  }

  @Override
  public CompletableFuture<Optional<BaseRequest<?, ?>>> process(
      CallbackQueryContext callbackQueryContext, Update update) {
    return CompletableFuture.supplyAsync(
        () ->
            Util.extractChat(callbackQueryContext, update)
                .map(
                    chat -> {
                      final String message = modelHelp.getMessage(chat, tgCommandProcessors);
                      final Optional<Integer> messageId = Util.extractMessageId(update);
                      final InlineKeyboardButton[] buttons =
                          modelHelp.generateInlineKeyBoardButton(chat, eventProcessor);
                      BaseRequest<?, ?> request;
                      if (messageId.isPresent()) {
                        final EditMessageText editMessageText =
                            new EditMessageText(chat.getId(), messageId.get(), message);
                        editMessageText.replyMarkup(new InlineKeyboardMarkup(buttons));
                        request = editMessageText;
                      } else {
                        final SendMessage sendMessage =
                            new SendMessage(chat.getId(), message).parseMode(ParseMode.Markdown);
                        sendMessage.replyMarkup(new InlineKeyboardMarkup(buttons));
                        request = sendMessage;
                      }

                      return request;
                    }),
        threadPool);
  }
}
