package com.github.polpetta.mezzotre.telegram.model;

import com.github.polpetta.mezzotre.i18n.TemplateContentGenerator;
import com.github.polpetta.mezzotre.orm.model.CallbackQueryContext;
import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.github.polpetta.mezzotre.telegram.callbackquery.Field;
import com.github.polpetta.mezzotre.telegram.command.Processor;
import com.github.polpetta.mezzotre.util.Clock;
import com.github.polpetta.mezzotre.util.UUIDGenerator;
import com.github.polpetta.types.json.CallbackQueryMetadata;
import com.github.polpetta.types.json.ChatContext;
import com.pengrad.telegrambot.model.request.InlineKeyboardButton;
import java.util.Map;
import java.util.stream.Collectors;
import javax.inject.Inject;
import org.apache.commons.lang3.tuple.Pair;

// FIXME tests!
public class Help {

  private static final String TRIGGERING_STAGING_NAME = "/help";

  private final TemplateContentGenerator templateContentGenerator;
  private final Clock clock;
  private final UUIDGenerator uuidGenerator;

  @Inject
  public Help(
      TemplateContentGenerator templateContentGenerator, Clock clock, UUIDGenerator uuidGenerator) {

    this.templateContentGenerator = templateContentGenerator;
    this.clock = clock;
    this.uuidGenerator = uuidGenerator;
  }

  public String getMessage(TgChat chat, Map<String, Processor> tgCommandProcessors) {
    final String message =
        templateContentGenerator.mergeTemplate(
            velocityContext -> {
              velocityContext.put(
                  "commands",
                  tgCommandProcessors.values().stream()
                      .distinct()
                      .map(
                          p ->
                              Pair.of(
                                  p.getTriggerKeywords().stream()
                                      .sorted()
                                      .collect(Collectors.toList()),
                                  p.getLocaleDescriptionKeyword()))
                      .collect(Collectors.toList()));
            },
            chat.getLocale(),
            "template/telegram/help.vm");

    // FIXME this shouldn't stay here. We need to move it into another method
    final ChatContext chatContext = chat.getChatContext();
    chatContext.setStage(TRIGGERING_STAGING_NAME);
    chatContext.setStep(0);
    chatContext.setPreviousMessageUnixTimestampInSeconds(clock.now());
    chat.setChatContext(chatContext);
    chat.setHasHelpBeenShown(true);
    chat.save();
    return message;
  }

  public InlineKeyboardButton[] generateInlineKeyBoardButton(
      TgChat chat,
      Map<String, com.github.polpetta.mezzotre.telegram.callbackquery.Processor> eventProcessors) {
    final String callBackGroupId = uuidGenerator.generateAsString();
    return eventProcessors.values().stream()
        .filter(
            com.github.polpetta.mezzotre.telegram.callbackquery.Processor
                ::canBeDirectlyInvokedByTheUser)
        .filter(e -> e.getPrettyPrintLocaleKeyName().isPresent())
        .map(
            eventProcessor -> {
              final CallbackQueryContext callbackQueryContext =
                  new CallbackQueryContext(
                      uuidGenerator.generateAsString(),
                      callBackGroupId,
                      new CallbackQueryMetadata.CallbackQueryMetadataBuilder()
                          .withEvent(eventProcessor.getEventName())
                          .withTelegramChatId(chat.getId())
                          .withAdditionalProperty(
                              Field.ShowHelp.InvokedFromHelpMessage.getName(), true)
                          .build());
              callbackQueryContext.save();

              return new InlineKeyboardButton(
                      templateContentGenerator.getString(
                          chat.getLocale(), eventProcessor.getPrettyPrintLocaleKeyName().get()))
                  .callbackData(callbackQueryContext.getId());
            })
        .toArray(InlineKeyboardButton[]::new);
  }
}
